from nbox.relics.relics_rpc_client import RelicStore_Stub
from nbox.relics.proto import relics_pb2, relics_rpc_pb2, common_pb2
from nbox.relics.client import Relics, UserAgentType
from nbox.relics.utils import get_relic_file, get_relics_stub, print_relics