# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/relics.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12proto/relics.proto\"Y\n\x0b\x42\x61\x63kendInfo\x12\x19\n\x07\x62\x61\x63kend\x18\x01 \x01(\x0e\x32\x08.Backend\x12\x17\n\x0fnbx_resource_id\x18\x02 \x01(\t\x12\x16\n\x0enbx_access_key\x18\x03 \x01(\t\"\xfd\x01\n\x05Relic\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ncreated_on\x18\x03 \x01(\x03\x12\x15\n\rlast_modified\x18\x04 \x01(\x03\x12\x0f\n\x07starred\x18\x05 \x01(\x08\x12\x0c\n\x04tags\x18\x06 \x03(\t\x12\x14\n\x0cworkspace_id\x18\x07 \x01(\t\x12\x12\n\npermission\x18\x08 \x03(\t\x12\x10\n\x08ui_field\x18\t \x03(\t\x12\x12\n\ncreated_by\x18\n \x01(\t\x12\x1a\n\x04\x61uth\x18\x0b \x01(\x0b\x32\x0c.BackendInfo\x12$\n\x0b\x62ucket_meta\x18\x64 \x01(\x0b\x32\x0f.BucketMetadata\"\xb9\x01\n\x0e\x42ucketMetadata\x12\x13\n\x0b\x62ucket_name\x18\x01 \x01(\t\x12\x0e\n\x06region\x18\x02 \x01(\t\x12\x19\n\x07\x62\x61\x63kend\x18\x03 \x01(\x0e\x32\x08.Backend\x12\x34\n\x0b\x62ucket_tags\x18\x04 \x03(\x0b\x32\x1f.BucketMetadata.BucketTagsEntry\x1a\x31\n\x0f\x42ucketTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8d\x04\n\tRelicFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ncreated_on\x18\x02 \x01(\x03\x12\x15\n\rlast_modified\x18\x03 \x01(\x03\x12\x0f\n\x07starred\x18\x04 \x01(\x08\x12\x0c\n\x04tags\x18\x05 \x03(\t\x12\x0c\n\x04size\x18\x06 \x01(\x03\x12\x0f\n\x07\x63omment\x18\x07 \x01(\t\x12\x10\n\x08username\x18\x08 \x01(\t\x12\"\n\x04type\x18\t \x01(\x0e\x32\x14.RelicFile.RelicType\x12\x14\n\x0cworkspace_id\x18\n \x01(\t\x12\x12\n\nrelic_name\x18\x0b \x01(\t\x12\x10\n\x08relic_id\x18\x0c \x01(\t\x12\x14\n\x0c\x63ontent_type\x18\r \x01(\t\x12\x10\n\x08\x64ownload\x18\x0e \x01(\x08\x12\x0b\n\x03url\x18\x64 \x01(\t\x12(\n\x07headers\x18\x65 \x03(\x0b\x32\x17.RelicFile.HeadersEntry\x12\"\n\x04\x62ody\x18\x66 \x03(\x0b\x32\x14.RelicFile.BodyEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tBodyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"7\n\tRelicType\x12\t\n\x05UNSET\x10\x00\x12\x08\n\x04\x46ILE\x10\x01\x12\n\n\x06\x46OLDER\x10\x02\x12\t\n\x05RELIC\x10\x03\"=\n\nRelicFiles\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x19\n\x05\x66iles\x18\x02 \x03(\x0b\x32\n.RelicFile*a\n\x07\x42\x61\x63kend\x12\t\n\x05UNSET\x10\x00\x12\x07\n\x03NBX\x10\x01\x12\n\n\x06\x41WS_S3\x10\x02\x12\x0b\n\x07GCP_GCS\x10\x03\x12\x0e\n\nAZURE_BLOB\x10\x04\x12\n\n\x06OCI_OB\x10\x05\x12\r\n\tDO_SPACES\x10\x06\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.relics_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BUCKETMETADATA_BUCKETTAGSENTRY._options = None
  _BUCKETMETADATA_BUCKETTAGSENTRY._serialized_options = b'8\001'
  _RELICFILE_HEADERSENTRY._options = None
  _RELICFILE_HEADERSENTRY._serialized_options = b'8\001'
  _RELICFILE_BODYENTRY._options = None
  _RELICFILE_BODYENTRY._serialized_options = b'8\001'
  _BACKEND._serialized_start=1148
  _BACKEND._serialized_end=1245
  _BACKENDINFO._serialized_start=22
  _BACKENDINFO._serialized_end=111
  _RELIC._serialized_start=114
  _RELIC._serialized_end=367
  _BUCKETMETADATA._serialized_start=370
  _BUCKETMETADATA._serialized_end=555
  _BUCKETMETADATA_BUCKETTAGSENTRY._serialized_start=506
  _BUCKETMETADATA_BUCKETTAGSENTRY._serialized_end=555
  _RELICFILE._serialized_start=558
  _RELICFILE._serialized_end=1083
  _RELICFILE_HEADERSENTRY._serialized_start=935
  _RELICFILE_HEADERSENTRY._serialized_end=981
  _RELICFILE_BODYENTRY._serialized_start=983
  _RELICFILE_BODYENTRY._serialized_end=1026
  _RELICFILE_RELICTYPE._serialized_start=1028
  _RELICFILE_RELICTYPE._serialized_end=1083
  _RELICFILES._serialized_start=1085
  _RELICFILES._serialized_end=1146
# @@protoc_insertion_point(module_scope)