"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import rsock_pb2
import typing

class RSockStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    Handshake: grpc.UnaryUnaryMultiCallable[
        rsock_pb2.HandshakeRequest,
        rsock_pb2.HandshakeResponse]

    Tunnel: grpc.StreamStreamMultiCallable[
        rsock_pb2.DataPacket,
        rsock_pb2.DataPacket]


class RSockServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def Handshake(self,
        request: rsock_pb2.HandshakeRequest,
        context: grpc.ServicerContext,
    ) -> rsock_pb2.HandshakeResponse: ...

    @abc.abstractmethod
    def Tunnel(self,
        request_iterator: typing.Iterator[rsock_pb2.DataPacket],
        context: grpc.ServicerContext,
    ) -> typing.Iterator[rsock_pb2.DataPacket]: ...


def add_RSockServicer_to_server(servicer: RSockServicer, server: grpc.Server) -> None: ...
