# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jobs/job.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..jobs import dag_pb2 as jobs_dot_dag__pb2
from ..common import common_pb2 as common_dot_common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0ejobs/job.proto\x12\x04jobs\x1a\x0ejobs/dag.proto\x1a\x13\x63ommon/common.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa0\x05\n\x03Job\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1a\n\x04\x63ode\x18\x05 \x01(\x0b\x32\x0c.common.Code\x12\"\n\x08resource\x18\x06 \x01(\x0b\x32\x10.common.Resource\x12\x32\n\rfeature_gates\x18\x07 \x03(\x0b\x32\x1b.jobs.Job.FeatureGatesEntry\x12$\n\x08schedule\x18\x0c \x01(\x0b\x32\x12.jobs.Job.Schedule\x12\x1a\n\x03\x64\x61g\x18\r \x01(\x0b\x32\r.jobs.dag.DAG\x12\x0e\n\x06paused\x18\x0e \x01(\x08\x12 \n\x06status\x18\x0f \x01(\x0e\x32\x10.jobs.Job.Status\x12 \n\x07k8sInfo\x18\x10 \x01(\x0b\x32\x0f.common.K8sInfo\x1a\x33\n\x11\x46\x65\x61tureGatesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1al\n\x08Schedule\x12)\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x63ron\x18\x03 \x01(\t\"\x9b\x01\n\x06Status\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07NOT_SET\x10\x01\x12\r\n\tPREPARING\x10\x02\x12\n\n\x06PAUSED\x10\x03\x12\r\n\tSCHEDULED\x10\x04\x12\n\n\x06\x41\x43TIVE\x10\x05\x12\t\n\x05\x45RROR\x10\x06\x12\r\n\tCOMPLETED\x10\x07\x12\x0c\n\x08\x41RCHIVED\x10\x08\x12\r\n\tCANCELLED\x10\t\x12\n\n\x06KILLED\x10\nJ\x04\x08\x0b\x10\x0c\x42=Z;github.com/NimbleBoxAI/jobs-architecture/jobs_pb/v3;jobs_pbb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'jobs.job_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z;github.com/NimbleBoxAI/jobs-architecture/jobs_pb/v3;jobs_pb'
  _JOB_FEATUREGATESENTRY._options = None
  _JOB_FEATUREGATESENTRY._serialized_options = b'8\001'
  _JOB._serialized_start=95
  _JOB._serialized_end=767
  _JOB_FEATUREGATESENTRY._serialized_start=442
  _JOB_FEATUREGATESENTRY._serialized_end=493
  _JOB_SCHEDULE._serialized_start=495
  _JOB_SCHEDULE._serialized_end=603
  _JOB_STATUS._serialized_start=606
  _JOB_STATUS._serialized_end=761
# @@protoc_insertion_point(module_scope)
